/* Title: Database example with Capsicum sandbox */

#include <sys/resource.h>
#include <sys/capability.h>

#include <err.h>
#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include <ksql.h>

int
main(void)
{
	struct ksql	*sql;
	struct ksqlstmt	*stmt;
	int64_t		 hits;
	cap_rights_t	 rights;

	/*
	 * We can't use any Capsicum here.
	 * The reason is that we don't know the files beforehand.
	 * So we can't work with pre-opened file descriptors.
	 */

	sql = ksql_alloc(NULL);
	ksql_open(sql, "simple.db");
	ksql_stmt_alloc(sql, &stmt, 
		"UPDATE hits SET hits=hits+1", 0);
	ksql_stmt_step(stmt);
	ksql_stmt_free(stmt);
	ksql_stmt_alloc(sql, &stmt, 
		"SELECT hits FROM hits", 0);
	ksql_stmt_step(stmt);
	hits = ksql_stmt_int(stmt, 0);
	ksql_stmt_free(stmt);
	ksql_free(sql);

	/* Now we can uselessly use Capsicum. */

	cap_rights_init(&rights, CAP_EVENT, 
		CAP_WRITE, CAP_FSTAT);
	if (cap_rights_limit(STDOUT_FILENO, &rights) < 0)
		err(EXIT_FAILURE, NULL);
	cap_rights_init(&rights, CAP_EVENT, 
		CAP_WRITE, CAP_FSTAT);
	if (cap_rights_limit(STDERR_FILENO, &rights) < 0)
		err(EXIT_FAILURE, NULL);
	cap_rights_init(&rights, CAP_EVENT, 
		CAP_READ, CAP_FSTAT);
	if (cap_rights_limit(STDIN_FILENO, &rights) < 0)
		err(EXIT_FAILURE, NULL);
	if (cap_enter())
		err(EXIT_FAILURE, NULL);

	puts("Status: 200 OK\r");
	puts("Content-Type: text/html\r");
	puts("\r");
	printf("Hello, world: %" PRId64 "\n", hits);
	return(EXIT_SUCCESS);
}
